////////////////////////////////////////
// Complementary Reimagined by EminGT //
////////////////////////////////////////

//Common//
#include "/lib/common.glsl"

//////////Fragment Shader//////////Fragment Shader//////////Fragment Shader//////////
#ifdef FRAGMENT_SHADER

noperspective in vec2 texCoord;

#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
    flat in float vlFactor;
#endif

//Uniforms//
uniform float viewWidth, viewHeight;

uniform sampler2D colortex4;

#ifdef SSAO
    uniform int frameCounter;

    uniform float far, near;
    uniform float aspectRatio;

    uniform mat4 gbufferProjection;

    uniform sampler2D depthtex0;
#endif

//Pipeline Constants//

//Common Variables//
#ifdef SSAO
    float farMinusNear = far - near;

    vec2 view = vec2(viewWidth, viewHeight);
#endif

//Common Functions//
#ifdef SSAO
    float GetLinearDepth(float depth) {
        return (2.0 * near) / (far + near - depth * farMinusNear);
    }

    vec2 OffsetDist(float x, int s) {
        float n = fract(x * 1.414) * 3.1415;
        return pow2(vec2(cos(n), sin(n)) * x / s);
    }

    float DoAmbientOcclusion(vec2 coord, float dither) {
		coord *= 2.0;
		coord += 0.5 / vec2(viewWidth, viewHeight);
		if (coord.x < 0.0 || coord.x > 1.0 || coord.y < 0.0 || coord.y > 1.0) return 1.0;

        float ao = 0.0;
        int samples = 12;

        #ifdef TAA
			dither = fract(dither + 1.61803398875 * mod(float(frameCounter), 3600.0));
        #endif
        
        float z0 = texture2D(depthtex0, coord).r;
        if(z0 >= 1.0 || z0 < 0.56) return 1.0;
        z0 = GetLinearDepth(z0);
        
        float sampleDepth = 0.0, angle = 0.0, dist = 0.0;
        float fovScale = gbufferProjection[1][1] / 1.37;
        float distScale = max(farMinusNear * z0 + near, 3.0);
        vec2 scale = vec2(0.2625, 0.525) * vec2(1.0 / aspectRatio, 1.0) * fovScale / distScale;

        for (int i = 1; i <= samples; i++) {
            vec2 offset = OffsetDist(i + dither, samples) * scale;
            if (i % 2 == 0) offset.y = -offset.y;

            vec2 coord1 = coord + offset;
            vec2 coord2 = coord - offset;

            sampleDepth = GetLinearDepth(texture2D(depthtex0, coord1).r);
            float aosample = farMinusNear * (z0 - sampleDepth) * 2.0;
            angle = clamp(0.5 - aosample, 0.0, 1.0);
            dist = clamp(0.5 * aosample - 1.0, 0.0, 1.0);

            sampleDepth = GetLinearDepth(texture2D(depthtex0, coord2).r);
            aosample = farMinusNear * (z0 - sampleDepth) * 2.0;
            angle += clamp(0.5 - aosample, 0.0, 1.0);
            dist += clamp(0.5 * aosample - 1.0, 0.0, 1.0);
            
            ao += clamp(angle + dist, 0.0, 1.0);
        }
        ao /= samples;
        
        return ao;
    }
#endif

//Includes//
#ifdef SSAO
    #include "/lib/util/dither.glsl"
#endif

//Program//
void main() {
	#ifdef SSAO
    	float ao = DoAmbientOcclusion(texCoord, Bayer64(gl_FragCoord.xy));
	#else
		float ao = 1.0;
	#endif
    
    #if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
        if (max(texCoord.x, texCoord.y) > 0.75) ao = vlFactor;
    #endif

	/* DRAWBUFFERS:4 */
	gl_FragData[0] = vec4(ao, 0.0, 0.0, 1.0);
}

#endif

//////////Vertex Shader//////////Vertex Shader//////////Vertex Shader//////////
#ifdef VERTEX_SHADER

noperspective out vec2 texCoord;

#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
    flat out float vlFactor;
#endif

//Uniforms//
#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
	uniform float viewWidth, viewHeight;
	
	uniform sampler2D colortex4;
#endif

//Attributes//

//Common Variables//

//Common Functions//

//Includes//

//Program//
void main() {
	gl_Position = ftransform();
	texCoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
		vlFactor = texelFetch(colortex4, ivec2(viewWidth-1, viewHeight-1), 0).r;
    #endif
}

#endif
