////////////////////////////////////////
// Complementary Reimagined by EminGT //
////////////////////////////////////////

//Common//
#include "/lib/common.glsl"

//////////Fragment Shader//////////Fragment Shader//////////Fragment Shader//////////
#ifdef FRAGMENT_SHADER

noperspective in vec2 texCoord;

flat in vec3 upVec, sunVec;

#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
    flat in float vlFactor;
#endif

//Uniforms//
uniform int isEyeInWater;
uniform int frameCounter;

uniform float far, near;
uniform float viewWidth, viewHeight;
uniform float blindness;
uniform float frameTimeCounter;

uniform vec3 fogColor;
uniform vec3 skyColor;
uniform vec3 cameraPosition;

uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex6;
uniform sampler2D depthtex0;
uniform sampler2D noisetex;

#ifdef SSAO
	uniform sampler2D colortex4;
#endif

#if REFLECTION_QUALITY >= 3
	uniform mat4 gbufferProjection;
	uniform mat4 gbufferModelView;
	
	uniform sampler2D colortex5;
#endif

#if CLOUD_QUALITY >= 0
	uniform sampler2D colortex2;
#endif
#if CLOUD_QUALITY >= 2
	uniform sampler2DShadow shadowtex0;
#endif

#ifdef TEMPORAL_FILTER
	uniform vec3 previousCameraPosition;

	uniform mat4 gbufferPreviousProjection;
	uniform mat4 gbufferPreviousModelView;
#endif

//Pipeline Constants//
const bool colortex0MipmapEnabled = true;

//Common Variables//
float SdotU = dot(sunVec, upVec);
float sunFactor = GetSunFactor(SdotU);
float sunVisibility = GetSunVisibility(SdotU);
float sunVisibility2 = sunVisibility * sunVisibility;
float shadowTime = GetShadowTime(sunVisibility);

#ifdef SSAO
	vec2 aoOffsets[4] = vec2[4](
		vec2( 1.0,  0.0),
		vec2( 0.0,  1.0),
		vec2(-1.0,  0.0),
		vec2( 0.0, -1.0)
	);
#endif

#ifdef OVERWORLD
	vec3 lightVec = sunVec * ((timeAngle < 0.5325 || timeAngle > 0.9675) ? 1.0 : -1.0);
#else
	vec3 lightVec = sunVec;
#endif

//Common Functions//
#ifdef SSAO
    float GetLinearDepth(float depth) {
        return (2.0 * near) / (far + near - depth * (far - near));
    }

	float GetAmbientOcclusion(float z) {
		float ao = 0.0;
		float tw = 0.0;
		float lz = GetLinearDepth(z);

		vec2 halfView = vec2(viewWidth, viewHeight) * 0.5;
		vec2 coord1 = (floor(texCoord * halfView + 1.0)) / halfView;
		vec2 coord2 = texCoord * 0.5;
		
		for (int i = 0; i < 4; i++) {
			vec2 offset = aoOffsets[i] / vec2(viewWidth, viewHeight);
			float samplez = GetLinearDepth(texture2D(depthtex0, coord1 + offset * 3.0).r);
			float wg = max(1.0 - 20.0 * far * abs(lz - samplez), 0.0);
			ao += texture2D(colortex4, coord2 + offset).r * wg;
			tw += wg;
		}
		ao /= tw;
		if (tw < 0.0001) ao = texture2D(colortex4, coord2).r;
		
		//return texture2D(colortex4, coord2).r;
		return clamp(pow1_5(ao), 0.0, 1.0);
	}
#endif

#if REFLECTION_QUALITY >= 3
	vec3 nvec3(vec4 pos) {
		return pos.xyz/pos.w;
	}

	vec4 nvec4(vec3 pos) {
		return vec4(pos.xyz, 1.0);
	}

	float cdist(vec2 coord) {
		return max(abs(coord.s-0.5) * 1.82, abs(coord.t-0.5) * 2.0);
	}
#endif

#ifdef TEMPORAL_FILTER
	// Previous frame reprojection from Chocapic13
	vec2 Reprojection(vec3 playerPos, vec3 cameraOffset) {
		vec4 proPos = vec4(playerPos + cameraOffset, 1.0);
		vec4 previousPosition = gbufferPreviousModelView * proPos;
		previousPosition = gbufferPreviousProjection * previousPosition;
		return previousPosition.xy / previousPosition.w * 0.5 + 0.5;
	}
#endif

//Includes//
#include "/lib/util/spaceConversion.glsl"
#include "/lib/util/dither.glsl"
#include "/lib/atmospherics/fog/mainFog.glsl"

#if REFLECTION_QUALITY >= 3
	#include "/lib/atmospherics/sky.glsl"
	#ifdef END
		#include "/lib/atmospherics/enderBeams.glsl"
	#endif
#endif

#if defined OVERWORLD && CLOUD_QUALITY > 0
	#include "/lib/atmospherics/volumetricClouds.glsl"
#endif

#ifdef END
	#include "/lib/atmospherics/enderStars.glsl"
#endif

//Program//
void main() {
	vec3 color = texelFetch(colortex0, texelCoord, 0).rgb;
	float z    = texelFetch(depthtex0, texelCoord, 0).r;

	vec4 screenPos = vec4(texCoord, z, 1.0);
	vec4 viewPos = gbufferProjectionInverse * (screenPos * 2.0 - 1.0);
	viewPos /= viewPos.w;
	float lViewPos = length(viewPos);
	vec3 nViewPos = normalize(viewPos.xyz);
	vec3 playerPos = ViewToPlayer(viewPos.xyz);

	float dither = texture2D(noisetex, texCoord * vec2(viewWidth, viewHeight) / 512.0).b;
	#ifdef TAA
		float ditherAnimate = 1.61803398875 * mod(float(frameCounter), 3600.0);
		dither = fract(dither + ditherAnimate);
	#endif

	float VdotU = dot(nViewPos, upVec);
	float VdotS = dot(nViewPos, sunVec);

	#ifdef TEMPORAL_FILTER
		vec4 refAndCloudNew = vec4(0.0);
	#endif

	#if defined OVERWORLD && CLOUD_QUALITY > 0
		bool sun = false;
	#endif
	if (z < 1.0) {
		vec3 texture5 = texelFetch(colortex1, texelCoord, 0).rgb;

		#ifdef SSAO
			float ssao = GetAmbientOcclusion(z);
		#else
			float ssao = 1.0;
		#endif
		
		int materialMaskInt = int(texture5.g * 255.1);
		bool intenseFresnel = false;
		float smoothnessD = texture5.r;
		vec3 reflectColor = vec3(1.0);
		
		#include "/lib/materials/deferredMaterials.glsl"
		
		color.rgb *= ssao;

		#if REFLECTION_QUALITY >= 3
			float skyLightFactor = texture5.b;
			vec3 normalM = mat3(gbufferModelView) * texelFetch(colortex5, texelCoord, 0).rgb;

			float fresnel = clamp(1.0 + dot(normalM, nViewPos), 0.0, 1.0);

			float fresnelFactor = (1.0 - smoothnessD) * 0.7;
			float fresnelM = max(fresnel - fresnelFactor, 0.0) / (1.0 - fresnelFactor);
			fresnelM = intenseFresnel ? fresnelM * 0.75 + 0.25 : pow2(fresnelM);
			fresnelM = fresnelM * smoothnessD - dither * 0.001;

			if (fresnelM > 0.0) {
				vec3 roughPos = playerPos + cameraPosition;
				roughPos *= 1000.0;
				vec2 roughCoord = roughPos.xz + roughPos.y;
				#ifdef TAA
					roughCoord += ditherAnimate;
				#endif
				vec3 roughNoise = texture2D(noisetex, roughCoord).rgb;
				roughNoise = vec3(0.3, 0.3, 0.3) * (roughNoise - vec3(0.5));
				
				roughNoise *= pow2(1.0 - smoothnessD);

				normalM += roughNoise;
			
				#include "/lib/materials/deferredReflections.glsl"

				//if (gl_FragCoord.x > 960) color = vec3(0.25,0,0.25);
			}
		#endif

		DoFog(color, lViewPos, playerPos, VdotU, VdotS, dither);
	} else { // Sky
		#if defined OVERWORLD && CLOUD_QUALITY > 0
			sun = color.r > 2.0;
		#endif
		#ifdef NETHER
			color.rgb = fogColor;
		#endif
		#ifdef END
			color.rgb = endSkyColor;
			color.rgb += GetEnderStars(viewPos.xyz, VdotU);
		#endif
	}

	float cloudLinearDepth = 1.0;
	#if defined OVERWORLD && CLOUD_QUALITY > 0
		vec4 volumetricClouds = vec4(0.0);
		if (z > 0.56) {
			float threshold1 = 800.0;
			#ifndef SECOND_CLOUD_LAYER
				volumetricClouds =
					GetVolumetricClouds(CLOUD_ALT1, threshold1, cloudLinearDepth, sun, playerPos, lViewPos, nViewPos, VdotS, VdotU, dither);
			#else
				float threshold2 = 1000.0;

				if (abs(cameraPosition.y - CLOUD_ALT1) < abs(cameraPosition.y - CLOUD_ALT2)) {
					volumetricClouds =
					GetVolumetricClouds(CLOUD_ALT1, threshold1, cloudLinearDepth, sun, playerPos, lViewPos, nViewPos, VdotS, VdotU, dither);
					if (volumetricClouds.a < 0.5) volumetricClouds =
					GetVolumetricClouds(CLOUD_ALT2, threshold2, cloudLinearDepth, sun, playerPos, lViewPos, nViewPos, VdotS, VdotU, dither);
				} else {
					volumetricClouds =
					GetVolumetricClouds(CLOUD_ALT2, threshold2, cloudLinearDepth, sun, playerPos, lViewPos, nViewPos, VdotS, VdotU, dither);
					if (volumetricClouds.a < 0.5) volumetricClouds =
					GetVolumetricClouds(CLOUD_ALT1, threshold1, cloudLinearDepth, sun, playerPos, lViewPos, nViewPos, VdotS, VdotU, dither);
				}
			#endif
		}

		#ifndef TEMPORAL_FILTER
			color = mix(color, volumetricClouds.rgb, volumetricClouds.a);
		#else
			refAndCloudNew.rgb += volumetricClouds.rgb * volumetricClouds.a;
			refAndCloudNew.a = volumetricClouds.a;
		#endif
	#endif
	#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
		if (viewWidth + viewHeight - gl_FragCoord.x - gl_FragCoord.y < 1.5)
			cloudLinearDepth = vlFactor;
	#endif

	#ifdef TEMPORAL_FILTER
		vec4 refAndCloudWrite;

		vec3 cameraOffset = (cameraPosition - previousCameraPosition) * float(z > 0.56);
		vec2 prvCoord = Reprojection(playerPos, cameraOffset);
		vec4 refAndCloudOld = texture2D(colortex6, prvCoord);
		if (z > 0.56) {
			float blendFactor = float(prvCoord.x > 0.0 && prvCoord.x < 1.0 && prvCoord.y > 0.0 && prvCoord.y < 1.0);
			float velocity = length(cameraOffset) * max(16.0 - lViewPos / gbufferProjection[1][1], 3.0);
			blendFactor *= max(exp(-velocity) * 0.95, 0.25);
			refAndCloudWrite = mix(refAndCloudNew, refAndCloudOld, blendFactor);

			if (texCoord.x < -0.5) {
				color.rgb *= 1.0 - refAndCloudNew.a;
				color.rgb += refAndCloudNew.rgb;
			} else {
				color.rgb *= 1.0 - refAndCloudWrite.a;
				color.rgb += refAndCloudWrite.rgb;
			}
		} else {
			refAndCloudWrite = refAndCloudOld;
		}
	#endif

	/*DRAWBUFFERS:045*/
    gl_FragData[0] = vec4(color, 1.0);
	gl_FragData[1] = vec4(cloudLinearDepth, 0.0, 0.0, 1.0);
	gl_FragData[2] = vec4(pow(color, vec3(0.125)) * 0.5, 1.0);
	#ifdef TEMPORAL_FILTER
		/*DRAWBUFFERS:0456*/
		gl_FragData[3] = refAndCloudWrite;
	#endif
}

#endif

//////////Vertex Shader//////////Vertex Shader//////////Vertex Shader//////////
#ifdef VERTEX_SHADER

noperspective out vec2 texCoord;

flat out vec3 upVec, sunVec;

#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
    flat out float vlFactor;
#endif

//Uniforms//
#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
	uniform float viewWidth, viewHeight;
	
	uniform sampler2D colortex4;
#endif

//Attributes//

//Common Variables//

//Common Functions//

//Includes//

//Program//
void main() {
	gl_Position = ftransform();
	texCoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	upVec = normalize(gbufferModelView[1].xyz);
	sunVec = GetSunVector();

	#if defined OVERWORLD && defined SCENE_AWARE_LIGHT_SHAFTS
		vlFactor = texelFetch(colortex4, ivec2(viewWidth-1, viewHeight-1), 0).r;
    #endif
}

#endif
